@@ -11,9 +11,9 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2000 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
 // Basic Hash Table implementation
 // Implementation
 
@@ -108,7 +108,7 @@ void* BasicHashTable::Iterator::next(char const*& key) {
 
   BasicHashTable::TableEntry* entry = fNextEntry;
   fNextEntry = entry->fNext;
-  
+
   key = entry->key;
   return entry->value;
 }
@@ -175,7 +175,11 @@ void BasicHashTable::assignKey(TableEntry* entry, char const* key) {
   } else if (fKeyType == ONE_WORD_HASH_KEYS) {
     entry->key = key;
   } else if (fKeyType > 0) {
-    entry->key = (char const*)(new unsigned[fKeyType]);
+    unsigned* keyFrom = (unsigned*)key;
+    unsigned* keyTo = new unsigned[fKeyType];
+    for (int i = 0; i < fKeyType; ++i) keyTo[i] = keyFrom[i];
+
+    entry->key = (char const*)keyTo;
   }
 }
 
@@ -259,10 +263,10 @@ unsigned BasicHashTable::hashIndexFromKey(char const* key) const {
     }
     result &= fMask;
   } else if (fKeyType == ONE_WORD_HASH_KEYS) {
-    result = randomIndex((unsigned long)key);
+    result = randomIndex((uintptr_t)key);
   } else {
     unsigned* k = (unsigned*)key;
-    unsigned long sum = 0;
+    uintptr_t sum = 0;
     for (int i = 0; i < fKeyType; ++i) {
       sum += k[i];
     }
@@ -271,4 +275,3 @@ unsigned BasicHashTable::hashIndexFromKey(char const* key) const {
 
   return result;
 }
-
